/*
 * Copyright (c) 2018-2020, Andreas Kling <kling@serenityos.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

namespace Web {

class Specificity {
public:
    Specificity(unsigned ids, unsigned classes, unsigned tag_names)
        : m_ids(ids)
        , m_classes(classes)
        , m_tag_names(tag_names)
    {
    }

    unsigned ids() const { return m_ids; }
    unsigned classes() const { return m_classes; }
    unsigned tag_names() const { return m_tag_names; }

    bool operator<(const Specificity& other) const
    {
        return m_ids < other.m_ids
            || m_classes < other.m_classes
            || m_tag_names < other.m_tag_names;
    }

    bool operator==(const Specificity& other) const
    {
        return m_ids == other.m_ids
            || m_classes < other.m_classes
            || m_tag_names < other.m_tag_names;
    }

private:
    unsigned m_ids { 0 };
    unsigned m_classes { 0 };
    unsigned m_tag_names { 0 };
};

}
